/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Mobility Components.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef BOARD_H
#define BOARD_H

#include <QObject>
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QGraphicsPixmapItem>
#include <QPixmap>
#include <QPropertyAnimation>

class Board : public QGraphicsView
{
    Q_OBJECT
    Q_PROPERTY(int connectRotation READ connectRotation WRITE setConnectRotation)
    Q_PROPERTY(qreal connectOpacity READ connectOpacity WRITE setConnectOpacity)
public:
    explicit Board(QWidget *parent = 0);

    enum Edge {
        Top = 1,
        Bottom,
        Left,
        Right
    };

    enum BoardSize {
        Width = 640,
        Height = 360,
        Paddle = 50
    };


    QGraphicsScene *getScene() {
        return scene;
    }

    int connectRotation(){
        return connectIcon->rotation();
    }

    qreal connectOpacity(){
        return connectIcon->opacity();
    }

    void setConnectRotation(int rot);
    void setConnectOpacity(qreal op);

signals:
    void ballCollision(Board::Edge pos);
    void scored(Board::Edge pos);

public slots:
    void setBall(int x, int y);
    void setLeftPaddle(int y);
    void setRightPaddle(int y);
    void setScore(int l, int r);

    void setStatus(QString text, int opacity_start, int opacity_end);

    void sceneChanged(const QList<QRectF> &region);

    void animateConnect(bool start = true);
    void fadeConnect(bool out = true);

private:
    QGraphicsScene *scene;
    QGraphicsRectItem *ball;
    QGraphicsRectItem *topWall;
    QGraphicsRectItem *bottomWall;
    QGraphicsRectItem *leftPaddle;
    QGraphicsRectItem *rightPaddle;

    QGraphicsTextItem *leftScore;
    QGraphicsTextItem *rightScore;

    QGraphicsTextItem *status;

    QPixmap icon;
    QGraphicsPixmapItem *connectIcon;
    QPropertyAnimation *connectAnimation;

    void checkBall(int x, int y);
};

#endif // BOARD_H
